/**************************************************************************/
/* Searches for text in a specified file range                            */
/**************************************************************************/
/*  2004-10-21 : Initial program release.                                 */
/**************************************************************************/
/* The latest version of this program can be found at                     */
/*               www.degeus.com/rexxseek                                  */
/**************************************************************************/
/* This program is released under the terms of the GNU license, see       */
/*               www.gnu.org/copyleft/gpl.html                            */
/**************************************************************************/
/* (c) 2004 Marcus C. de Geus                                             */
/*          marcus@degeus.com                                             */
/*          www.degeus.com                                                */
/**************************************************************************/
/* Use it if you like it. Don't if you don't. No legalese.                */
/**************************************************************************/

signal on Halt  /* handle halt */

parse source . . ProgPath  /* get the program path */

ProgDir = filespec('D',ProgPath)||filespec('P',ProgPath)  /* the program dir */
ProgName = left(filespec('N',ProgPath),lastpos('.',filespec('N',ProgPath)))  /* the program name plus last dot */

if (\LoadRexxUtil()) then  /* if we cannot get the REXX utilities */
do
 call Halt  /* quit */
end

parse arg Arguments  /* get the arguments */

Arguments = strip(Arguments)  /* remove any excess spaces */

if (left(Arguments,1) >< '/') then  /* if the arguments do not start with a slash */
do
 call Usage  /* show them how to do it */
end

Bare = 0  /* not bare */
EndDT = ''  /* no end time yet */
ErrorCode = 0  /* OK */
ExeName = ''  /* start with nothing */
SearchCount = 0  /* nothing yet */
FileName = ''  /* start with nothing */
FolderID = ''  /* start with nothing */
Hits = 0  /* start with nothing */
MaxObjects = 0  /* nothing yet */
NamePart = ''  /* nothing yet */
NewExample = ''  /* start with nothing */
Objects = 0  /* nothing yet */
OpenRes = 0  /* no need to open anything yet */
Parameters = ''  /* start with nothing */
Quiet = 0  /* not quiet */
TempDir = ''  /* no temp dir yet */
RemTempDir = 0  /* no need to remove the temp dir yet */
ResultsDir = ''  /* start with nothing */
Text = ''  /* start with nothing */
StartDT = GetDT()  /* the start time */
SubDirs = 0  /* no recursing */

Example = ProgDir||ProgName||'FLD'  /* the default folder template */

if (\CheckDir(Example,0)) then  /* if it does not check out -- no warning */
do
 Example = ''  /* use nothing (yet) */
end

Arguments = strip(Arguments,'L','/')  /* remove the leading slash */

do while (Arguments >< '')  /* as long as we have arguments left */

 parse var Arguments NextArg '/' Arguments  /* separate the first argument string */
 parse var NextArg ArgName +1 . +1 ArgValue  /* get the argument type and value, but lose the separator */

 ArgName = translate(ArgName)  /* make this bit uppercase */
 ArgValue = strip(strip(ArgValue),'B','"')  /* remove any excess space and double quotation marks from this bit */

 select  /* do one of the following */

  when ((ArgName = 'B') & (ArgValue = '')) then  /* if we have a bare action argument */
  do
   Bare = 1  /* use this */
  end

  when ((ArgName = 'C') & (ExeName = '')) then  /* if we have a fresh command argument */
  do
   ExeName = ArgValue  /* got it */
  end

  when ((ArgName = 'E') & (NewExample = '')) then  /* if we have a fresh example folder argument */
  do
   NewExample = ArgValue  /* got it */
  end

  when ((ArgName = 'F') & (FileName = '')) then  /* if we have a fresh file spec argument */
  do
   FileName = ArgValue  /* got it */
  end

  when ((ArgName = 'H') | (ArgName = '?')) then  /* if we have a help argument */
  do
   call Usage  /* show them how to do it */
  end

  when ((ArgName = 'M') & (MaxObjects = 0) & (datatype(ArgValue,'W'))) then  /* if we have a fresh whole number maximum objects argument */
  do
   MaxObjects = ArgValue  /* got it */
  end

  when ((ArgName = 'N') & (NamePart = '')) then  /* if we have a fresh partial file name argument */
  do
   NamePart = ArgValue  /* got it */
  end

  when ((ArgName = 'O') & (ArgValue = '')) then  /* if we have an open folders argument */
  do
   OpenRes = 1  /* use this */
  end

  when ((ArgName = 'P') & (Parameters = '')) then  /* if we have a fresh parameters argument */
  do
   Parameters = ArgValue  /* we have a parameters string */
  end

  when ((ArgName = 'Q') & (ArgValue = '')) then  /* if we have a quiet argument */
  do
   Quiet = 1  /* got it */
  end

  when ((ArgName = 'R') & (ResultsDir = '')) then  /* if we have a fresh results dir argument */
  do
   ResultsDir = ArgValue  /* got it */
  end

  when ((ArgName = 'S') & (ArgValue = '')) then  /* if we have a subdirs argument */
  do
   SubDirs = 1  /* got it */
  end

  when ((ArgName = 'T') & (Text = '')) then  /* if we have a fresh text argument */
  do
   Text = ArgValue  /* got it */
  end

  when (ArgName = 'X')  then  /* if we have an examples argument */
  do
   call Examples  /* show the examples */
  end

  otherwise  /* if none of the above */
  do
   call Usage  /* show them how to do it */
  end

 end

end

if (\Bare) then  /* unless we are running bare */
do

 if (ResultsDir = '') then  /* if no results folder is specified */
 do
  ResultsDir = '<WP_DESKTOP>'  /* use the desktop */
 end
 else  /* if we have a results dir */
 do
  ResultsDir = strip(ResultsDir,'T','\')  /* remove any trailing backslash */
  call CheckDir ResultsDir,1  /* if it does not check out, report an error and quit */
 end

 if (NewExample ><  '') then  /* if we have a new folder template name */
 do
  NewExample = strip(NewExample,'T','\')  /* remove any trailing backslash */
  call CheckDir NewExample,1  /* if it does not check out, report an error and quit */
  Example = NewExample  /* otherwise use it */
 end

 if (Example >< '') then  /* if we have a folder template */
 do

  TempDir = strip(value('TEMP',,'OS2ENVIRONMENT'),'T','\')  /* look for the OS/2 temp dir and remove any trailing backslash */

  if ((TempDir = '') | (translate(TempDir) = translate(ResultsDir)) | (translate(TempDir) = translate(Example))) then  /* if we have a location problem */
  do

   TempDir = directory()  /* use the current working dir */

   if (length(TempDir) = 3) then  /* if we get a root dir */
   do

    TempDir = TempDir||'RexxSeek.TMP'  /* use this */

    if (\CheckDir(TempDir,0)) then  /* if it does not exist */
    do

     if (sysmkdir(TempDir) >< 0) then  /* if we cannot create it */
     do
      call GotError 'Cannot create temporary directory "'||TempDir||'"'  /* report and quit */
     end

     RemTempDir = 1  /* remove this temp dir after use */

    end

   end

  end

 end

 FolderID = CreateFolder(ResultsDir,Example,TempDir)  /* create the results folder */

 if (RemTempDir) then  /* if we want the temp dir removed */
 do
  call sysrmdir TempDir  /* remove the temp dir */
 end

end

if (FileName = '') then  /* if no file name is specified */
do
 FileName = strip(directory(),'T','\')||'\*'  /* use all files in the current dir */
end

if (NamePart >< '') then  /* if we have a name part */
do
 FileName = strip(strip(FileName,'T','*'),'T','\')||'\*'||NamePart||'*'  /* add it to any dir spec we may already have */
end

if (\Quiet) then  /* unless we are running quietly */
do

 if (\Bare) then  /* unless we are running bare */
 do

  say 'Results folder ID = '||FolderID  /* report */

  ReportText = 'Results folder will be created '  /* start with this */

  if (ResultsDir = '<WP_DESKTOP>') then  /* if we are storing on the desktop */
  do
   ReportText = ReportText||'on the desktop'  /* add this */
  end
  else  /* if elsewhere */
  do
   ReportText = ReportText||'in '||ResultsDir  /* add this */
  end

  say ReportText  /* report */

 end

 ReportText = 'Looking for '  /* start with this */

 if (Text >< '') then  /* if we have a search string */
 do
  ReportText = ReportText||'"'||Text||'" in '  /* add this */
 end

 ReportText = ReportText||FileName  /* add this */

 if (SubDirs) then  /* if we will be looking in subdirs as well */
 do
  ReportText = ReportText||' (recursive)'  /* add this */
 end

 say ReportText  /* report */

 if (MaxObjects > 0) then  /* if we have a maximum hit count */
 do
  say 'Maximum number of result objects = '||MaxObjects  /* add this */
 end
 
end

if (\Bare) then  /* unless we are running bare */
do

 call UpdateFolder FolderID,Text,FileName,SubDirs,,SearchCount,MaxObjects,Hits,StartDT  /* update the results folder */

 if (OpenRes) then  /* if we want the results folder opened */
 do
  call sysopenobject FolderID,0,1  /* open the results folder */
 end

end

Options = 'FO'  /* start with bare file specs */

if (SubDirs) then  /* if we want to look in subdirs as well */
do
 Options = Options||'S'  /* add an S */
end

call sysfiletree FileName,'Files.',Options  /* look for matching files */

if (Files.0 > 0) then  /* if we have one or more files */
do
 CountLength = length(Files.0)  /* get the number of digits in the results count */
 Indent = copies(' ',(CountLength * 2))||'  '  /* the indent to use */
end

do Index = 1 to Files.0  /* take each file in turn */

 if (\Quiet) then  /* unless we are running quietly */
 do
  say right(Index,CountLength,'0')||'/'||Files.0||' '||Files.Index  /* report */
 end

 if (Text = '') then  /* if we have no search string */
 do
  GotIt = 1  /* we always get what we want, since the contents don't matter */
 end

 else  /* if we have a search string */

 do

  if (verify(Text,xrange('A','Z'),'MATCH') > 0) then  /* if the search string contains one or more uppercase characters */
  do
   Options = 'C'  /* do a case-sensitive sysfilesearch below */
  end
  else  /* if we have only lowercase characters */
  do
   Options = ''  /* no option required */
  end

  if (\Quiet) then  /* unless we are running quietly */
  do
   Options = Options||'N'  /* include line numbers */
  end

  SearchRes = sysfilesearch(Text,Files.Index,'Found.',Options)  /* try to look for our stuff */

  if (SearchRes = 0) then  /* if the search succeeded */
  do

   SearchCount = SearchCount + 1  /* up the searched files counter */
   GotIt = (Found.0 > 0)  /* if we found the text, we have what we want */

   if (\Quiet) then  /* unless we are running quietly */
   do

    GotLength = length(Found.0)  /* get the number of digits */

    do SubIndex = 1 to Found.0  /* take each hit line */
     LineNo = right(word(Found.SubIndex,1),GotLength)  /* format the line number */
     TextLine = translate(subword(Found.SubIndex,2),copies('_',32),xrange('00'x,'1f'x))  /* remove awkward characters from the text */
     say Indent||LineNo||' : '||TextLine  /* and show the combo */
    end

   end

  end
  else  /* if not */
  do

   if (SearchRes = 2) then  /* if this came back */
   do
    say Indent||'Error : not enough memory to search file "'||Files.Index||'"'  /* report */
   end
   else  /* otherwise */
   do
    say Indent||'Error : could not open file "'||Files.Index||'"'  /* report */
   end

  end

 end

 if (GotIt) then  /* if we got what we were looking for */
 do

  Hits = Hits + 1  /* up the hits counter */

  if (\Bare) then  /* unless we are operating bare */
  do

   if (ExeName >< '') then  /* if we have a program to run */
   do
    Title = 'Open '||filespec('D',Files.Index)||filespec('P',Files.Index)||filespec('N',Files.Index)  /* a program object title */
    call syscreateobject 'WPProgram',Title,FolderID,'EXENAME='||ExeName||';PARAMETERS='||Parameters||' "'||Files.Index||'";','REPLACE'  /* make the object */
   end
   else  /* if not */
   do
    call syscreateshadow Files.Index,FolderID  /* make a shadow of the object in the results dir */
   end

   if (\Quiet) then  /* unless we are running quietly */
   do
    say Indent||'Shadow or program object created'  /* report */
   end

   if (Hits = MaxObjects) then  /* if we have reached a maximum */
   do

    if (\Quiet) then  /* unless we are running quietly */
    do
     say 'Maximum number of objects reached; discontinuing search.'  /* report */
    end

    leave  /* exit the loop */

   end

  end

 end

 if (\Bare) then  /* unless we are operating bare */
 do
  call UpdateFolder FolderID,Text,FileName,SubDirs,Files.0,SearchCount,MaxObjects,Hits,StartDT  /* update the results folder */
 end

end

if (\Bare) then  /* unless we are running bare */
do
 EndDT = GetDT()  /* the end time */
 call UpdateFolder FolderID,Text,FileName,SubDirs,Files.0,SearchCount,MaxObjects,Hits,StartDT,EndDT  /* update the results folder */
end

if (\Quiet) then  /* unless we are running quietly */
do

 say 'Number of files found     : '||Files.0  /* report */

 if (Text >< '') then  /* if we had a search string */
 do
  say 'Number of files searched  : '||SearchCount  /* report */
  say 'Number of search hits     : '||Hits  /* report */
 end
 else  /* if we were looking for files */
 do
  say 'Number of matching files  : '||Hits  /* report */
 end

 if (\Bare) then  /* unless we are running bare */
 do
  say 'Number of objects created : '||Hits  /* report */
 end

end

call Halt  /* that's all, folks! */

/**************************************************************************/
CreateFolder: procedure  /* creates a results folder */
/**************************************************************************/

parse arg ResultsDir,Example,TempDir  /* get the arguments */

FolderID = '<REXXSEEK_'||translate(time('L'),'_',':')||'>'  /* a reasonably unique object ID */

if ((Example >< '') & (TempDir >< '')) then  /* if we have a folder template and a temp dir */
do

 if (\syscopyobject(Example,TempDir)) then  /* if we cannot copy the template dir to the temp dir */
 do
  call GotError 'Cannot copy template folder to "'||TempDir||'"'  /* report & quit */
 end

 if (\syssetobjectdata(TempDir||'\'||filespec('N',Example),'OBJECTID='FolderID||';TITLE='||FolderID)) then  /* if we cannot set the object ID and title */
 do
  call GotError 'Cannot set title of results folder'  /* report & quit */
 end

 if (\sysmoveobject(FolderID,ResultsDir)) then  /* if we cannot move the folder to our results base folder */
 do
  call GotError 'Cannot move results folder to "'||ResultsDir||'"'  /* report & quit */
 end

end
else  /* if we have no folder template */
do

 if (\syscreateobject('WPFolder',FolderID,ResultsDir,'OBJECTID='||FolderID,'FAIL')) then  /* if we cannot create it */
 do
  call GotError 'Cannot create folder "'||FolderID||'"'  /* report and quit */
 end

end

return FolderID  /* end of CreateFolder */

/**************************************************************************/
UpdateFolder: procedure  /* updates a results folder */
/**************************************************************************/

parse arg FolderID,Text,FileName,SubDirs,Files,Searched,MaxObjects,Hits,StartDT,EndDT  /* get the arguments */

LF = d2c(10)  /* define LF */

if (EndDT = '') then  /* if we have no end time */
do
 Title = 'Searching for '  /* start a title */
end
else  /* if we have an end time */
do
 Title = 'Search results for '  /* start a title */
end

if (Text >< '') then  /* if we had a search string */
do
 Title = Title||'"'||Text||'"'||LF||'in '  /* add this */
end

Title = Title||FileName  /* add this */

if (SubDirs) then  /* if we recursed */
do
 Title = Title||' (recursive)'  /* add this */
end

if (Files = '') then  /* if we are still counting */
do
 Title = Title||LF||'Looking for files...'  /* add this */
end
else  /* if we have a file count */
do

 Title = Title||LF||'Files: '||Files  /* add this */

 if (Text >< '') then  /* if we have a search string */
 do
  Title = Title||' - Searched: '||Searched||' - Hits: '||Hits  /* add this */
 end
 else  /* if we are looking for files */
 do
  Title = Title||' - Objects: '||Hits  /* add this */
 end

 if (MaxObjects > 0) then  /* if we have a maximum hit number */
 do

  if (EndDT = '') then  /* if we are still searching */
  do
   Title = Title||' (max. '||MaxObjects||')'  /* add this */
  end
  else  /* if the search has ended */
  do

   if (Searched < Files) then  /* if we ended the search prematurely */
   do
    Title = Title||' (discontinued)'  /* add this */
   end

  end

 end

end

Title = Title||LF||'Start: '||StartDT  /* add this */

if (EndDT >< '') then  /* if we have an end time */
do
 Title = Title||LF||'End: '||EndDT  /* add this */
end

call syssetobjectdata FolderID,'TITLE='||Title  /* adjust the title */

return 1  /* end of UpdateFolder */

/**************************************************************************/
GetDT: procedure  /* returns date/time stamp */
/**************************************************************************/

DTStamp = left(date('S'),4)||'-'||substr(date('S'),5,2)||'-'||right(date('S'),2)||' - '||left(time('L'),11)  /* use this */

return DTStamp  /* end of GetDT */

/**************************************************************************/
CheckDir: procedure  /* checks the existence of a dir */
/**************************************************************************/

parse arg DirSpec,Warn  /* get the argument */

if ((filespec('D',DirSpec) = '') | (length(DirSpec) < 4)) then  /* if we get an incomplete spec or a root spec */
do
 call GotError 'Invalid folder name : '||DirSpec  /* report & quit */
end

Warn = (Warn = 1)  /* 1 = TRUE */

call sysfiletree DirSpec,'Dirs.','DO'  /* look for it */
DirExist = (Dirs.0 = 1)  /* if it exists, and it is the only one, set a flag */

if ((Warn) & (\DirExist)) then  /* if we did not find it, and we want a warning */
do
 call GotError 'Dir not found : '||DirSpec  /* report & quit */
end

return DirExist  /* end of CheckDir */

/**************************************************************************/
LoadRexxUtil: procedure  /* loads the Rexx utility functions library */
/**************************************************************************/

Result = 1  /* start with a good result */

if rxfuncquery('SysLoadFuncs') then  /* if it is not already loaded */
do

 if (rxfuncadd('SysLoadFuncs','RexxUtil','SysLoadFuncs') >< 0) then  /* if we cannot add it */
 do
  call GotError 'Cannot register Rexx Utilities library'  /* report & quit */
 end
 else  /* if we can add it */
 do

  if (SysLoadFuncs() = 0) then  /* if we cannot load it */
  do
   call GotError 'Cannot load Rexx Utilities library'  /* report & quit */
  end

 end

end

return Result  /* end of LoadRexxUtil */

/**************************************************************************/
Usage:  /* shows how it should be done */
/**************************************************************************/

CRLF = d2c(13)||d2c(10)  /* define CRLF */

Message = 'Usage : '||strip(ProgName,'T','.')||' [options]'||CRLF||,  /* report */
          '        Looks for text in a file range and displays the results in a folder'||CRLF||,  /* report */
          '        in the form of program objects or shadows.'||CRLF||,  /* report */
          CRLF||,  /* report */
          'Options:'||CRLF||,  /* report */
          CRLF||,  /* report */
          '/b               : Do not create results folders or objects (bare)'||CRLF||,  /* report */
          '/c <command>     : The command to include in program objects'||CRLF||,  /* report */
          '                     (none means create shadows instead)'||CRLF||,  /* report */
          '/e <dirspec>     : The folder to use as example for the results folders'||CRLF||,  /* report */
          '                     (none means '||ProgName||'FLD in the program directory)'||CRLF||,  /* report */
          '/f <filespec>    : The files to look for/in'||CRLF||,  /* report */
          '                     (none means all files in current dir)'||CRLF||,  /* report */
          '/h or /?         : Show this help text'||CRLF||,  /* report */
          '/m <number>      : Maximum number of objects to create'||CRLF||,  /* report */
          '                     (none means create objects for all hits)'||CRLF||,  /* report */
          '/n <namepart>    : A partial file name to narrow down the search'||CRLF||,  /* report */
          '                     (none means files specified by /f)'||CRLF||,  /* report */
          '/o               : Open results folders'||CRLF||,  /* report */
          '/p <parameters>  : The parameters to include in program objects'||CRLF||,  /* report */
          '                     (the result file name gets appended after a space)'||CRLF||,  /* report */
          '/q               : No messages (quiet)'||CRLF||,  /* report */
          '/r <dirspec>     : The folder in which to put the results folders'||CRLF||,  /* report */
          '                     (none means desktop)'||CRLF||,  /* report */
          '/s               : Recurse into subdirs'||CRLF||,  /* report */
          '/t <text>        : Text to look for in files'||CRLF||,  /* report */
          '                     (none means include all files)'||CRLF||,  /* report */
          '                     (any uppercase character means case-sensitive search)'||CRLF||,  /* report */
          '/x               : Show a few examples'  /* report */

call GotError Message  /* report an error & quit */

/**************************************************************************/
Examples:  /* shows a few examples of usage */
/**************************************************************************/

CRLF = d2c(13)||d2c(10)  /* define CRLF */
Prog = strip(ProgName,'T','.')  /* lose the trailing dot */

Message = 'Examples:'||CRLF||,  /* report */
          CRLF||,  /* report */
          'For up to 5 files in x:\*abc* containing the text "rexx" (case-INsensitive),'||CRLF||,  /* report */
          'create a shadow in a new results folder in y:\path:'||CRLF||,  /* report */
          CRLF||,  /* report */
          '    '||Prog||' /m=5 /f=x:\*abc* /t=rexx /r=y:\path'||CRLF||,  /* report */
          CRLF||,  /* report */
          'For each file in x:\*abc* containing the text "rexx" (case-INsensitive),'||CRLF||,  /* report */
          'create a program object for MyProg.exe in a new results folder in y:\path:'||CRLF||,  /* report */
          CRLF||,  /* report */
          '    '||Prog||' /f=x:\*abc* /t=rexx /c=MyProg.exe /r=y:\path'||CRLF||,  /* report */
          CRLF||,  /* report */
          'For each file in x:\*abc* containing the text "Rexx" (case-sensitive), do not'||CRLF||,  /* report */
          'create a results folder, and log the results in z:\logdir\rexxseek.log:'||CRLF||,  /* report */
          CRLF||,  /* report */
          '    '||Prog||' /f=x:\*abc* /t=Rexx /b > z:\logdir\rexxseek.log'||CRLF||,  /* report */
          CRLF||,  /* report */
          'For each file in x:\path with "abc" in its file name and containing the text'||CRLF||,  /* report */
          '"Rexx" (case-sensitive), create a shadow in a new results folder in y:\path'||CRLF||,  /* report */
          '(note that "/f=x:\path /n=abc" is equivalent to "/f=x:\path\*abc*"; this option'||CRLF||,  /* report */
          'is included for added drag & drop flexibility):'||CRLF||,  /* report */
          CRLF||,  /* report */
          '    '||Prog||' /f=x:\path /n=abc /t=Rexx /r=y:\path'  /* report */

say Message  /* report */

call Halt  /* quit */

/**************************************************************************/
GotError:  /* report an error and quits */
/**************************************************************************/

parse arg Message  /* get the argument */

call beep 666,111  /* signal */

say Message  /* report */

ErrorCode = 1  /* not OK */

call Halt  /* and quit */

/**************************************************************************/
Halt:  /* handles halt */
/**************************************************************************/

exit ErrorCode  /* just quit */
