/**************************************************************************/
/* Installs sample program objects for RexxSeek                           */
/**************************************************************************/

signal on Halt  /* handle halt */

parse source . . ProgPath  /* get the program path */

if (\LoadRexxUtil()) then  /* if we cannot get the REXX utilities */
do
 call Halt  /* quit */
end

ProgDir = directory()  /* get the program dir */

call sysfiletree ProgDir||'\rexxseek.cmd','Files.','FO'  /* look for RexxSeek.CMD */

if (Files.0 = 0) then  /* if we did not find it */
do
 call GotError 'This program must be started from the RexxSeek program directory.'  /* report and quit */
end

say 'This program installs a number of sample program objects for RexxSeek.CMD.'
say 'Hit [Enter] to continue, or [Ctrl][C],[Enter] to abort.'

pull  /* wait for [Enter] */

FolderID = '<RexxSeek_Objects>'  /* the RexxSeek folder ID */

call sysdestroyobject FolderID  /* remove any existing folder */

if (\SysCreateObject('WPFolder','RexxSeek','<WP_DESKTOP>','OBJECTID='||FolderID,'Update')) then  /* if we cannot create the objects folder */
do
 call GotError 'ERROR: cannot create objects folder.'  /* report and quit */
end

say 'RexxSeek object folder created.'  /* report */

ExeName = ProgDir||'\RexxSeek.CMD;'  /* the program to use for the next couple of objects */
LF = d2c(10)  /* define LF */

Title = 'RexxSeek'||LF||'Look for Files'||LF||'(recursive)'  /* the program object title */
ObjectID = '<RexxSeek_LookForFiles>'  /* the program object ID */
Parameters = '/s /f=[Files to look for:]'  /* the program object parameters */

call CreateObject ProgDir,FolderID,Title,ObjectID,ExeName,Parameters,1  /* try to create the object */

Title = 'RexxSeek'||LF||'Look for a String in Specified Files'||LF||'(recursive)'  /* the program object title */
ObjectID = '<ReexSeek_LookForStringInFiles>'  /* the program object ID */
Parameters = '/s /f=[Files to look in:] /t=[Text to look for:]'  /* the program object parameters */

call CreateObject ProgDir,FolderID,Title,ObjectID,ExeName,Parameters,1  /* try to create the object */

Title = 'RexxSeek'||LF||'Look for Files'||LF||'(recursive - bare)'||LF||'and View Results File'  /* the program object title */
ObjectID = '<RexxSeek_LookForFiles_Bare>'  /* the program object ID */
Parameters = '/b /s /f=[Files to look for:] > RexxSeek.RES & e RexxSeek.RES'  /* the program object parameters */

call CreateObject ProgDir,FolderID,Title,ObjectID,ExeName,Parameters,1  /* try to create the object */

Title = 'RexxSeek'||LF||'Look for a String in Specified Files'||LF||'(recursive - bare)'||LF||'and View Results File'  /* the program object title */
ObjectID = '<ReexSeek_LookForStringInFiles_Bare>'  /* the program object ID */
Parameters = '/b /s /f=[Files to look in:] /t=[Text to look for:] > RexxSeek.RES & e RexxSeek.RES'  /* the program object parameters */

call CreateObject ProgDir,FolderID,Title,ObjectID,ExeName,Parameters,1  /* try to create the object */

say 'Installation completed.'  /* report */
say 'Hit [Enter] to open the RexxSeek objects folder and quit.'  /* report */

pull  /* wait for [Enter] */

call sysopenobject FolderID,0,1  /* open the folder on the desktop */

call Halt  /* that's all, folks! */

/**************************************************************************/
CreateObject: procedure  /* creates a program object */
/**************************************************************************/

parse arg ProgDir,FolderID,Title,ObjectID,ExeName,Parameters,Minimized  /* get the arguments */

Minimized = (Minimized = 1)  /* 1 is true */

Values = 'OBJECTID='||ObjectID||';'||,  /* the object ID */
         'EXENAME='||ExeName||';'||,  /* the prog spec */
         'PARAMETERS='||Parameters||';'||,  /* the program parameters */
         'STARTUPDIR='||ProgDir||';'||,  /* the program dir */
         'NOAUTOCLOSE=NO;'||,  /* close on exit */
         'MINIMIZED='||subword('NO YES',(Minimized + 1))||';'  /* run invisible if necessary */

if (\SysCreateObject('WPProgram',Title,FolderID,Values,'Update')) then  /* if we cannot create the object */
do
 call GotError 'ERROR: Cannot create program object : '||ObjectID  /* report and quit */
end

say 'Program object created : '||ObjectID  /* report */

return 1  /* end of CreateObject */

/**************************************************************************/
LoadRexxUtil: procedure  /* loads the Rexx utility functions library */
/**************************************************************************/

Result = 1  /* start with a good result */

if rxfuncquery('SysLoadFuncs') then  /* if it is not already loaded */
do

 if (rxfuncadd('SysLoadFuncs','RexxUtil','SysLoadFuncs') >< 0) then  /* if we cannot add it */
 do
  call GotError 'Cannot register Rexx Utilities library'  /* report & quit */
 end
 else  /* if we can add it */
 do

  if (SysLoadFuncs() = 0) then  /* if we cannot load it */
  do
   call GotError 'Cannot load Rexx Utilities library'  /* report & quit */
  end

 end

end

return Result  /* end of LoadRexxUtil */

/**************************************************************************/
GotError:  /* beep and quit */
/**************************************************************************/

parse arg Message  /* get the error message */

say Message  /* report */

call beep 666,333  /* signal */
call beep 333,111  /* signal */

call Halt  /* and quit */

/**************************************************************************/
Halt:  /* we end up here */
/**************************************************************************/

exit  /* just quit */